/*
  A list of per-repo settings that override global settings.

*/
function GitFusionRepoGlobalOverrides(data) {

    Object.defineProperties(this, {
        /*
          Defines the default Unicode setting that Git Fusion applies to new repos.
          This setting is valid only when Git Fusion interacts with a Unicode-enabled Perforce server.

          (Defaults to `UTF-8`).

        */
        "charset": {
            value: data ? data.charset : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Allow Git users to create new repos by pushing/pulling a git url which specifies a Perforce depot path.
          This is similar to creating a repo from a p4 client.

        */
        "depotPathRepoCreationEnable": {
            value: data ? data.depotPathRepoCreationEnable : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Restrict which authenticated Git pushers are allowed to create new repos when depot-path-repo-creation-enable is enabled.

        */
        "depotPathRepoCreationP4group": {
            value: data ? data.depotPathRepoCreationP4group : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Defines whether Git Fusion assigns either the Git commit author or the Git pusher as the owner of a pushed change (submit).

        */
        "changeOwner": {
            value: data ? data.changeOwner : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Defines whether Git Fusion creates a new branch of Perforce depot file hierarchy for each copied branch of Git workspace history, including Git task branches as Git Fusion anonymous branches.

        */
        "enableGitBranchCreation": {
            value: data ? data.enableGitBranchCreation : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Permits branch creation for Swarm reviews, even when enable-git-branch-creation is disabled.

        */
        "enableSwarmReviews": {
            value: data ? data.enableSwarmReviews : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Defines whether Git Fusion copies merge commits and displays them in Perforce as integrations between Perforce branches.

        */
        "enableGitMergeCommits": {
            value: data ? data.enableGitMergeCommits : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Defines whether Git Fusion allows Git submodules to be pushed to Perforce.

        */
        "enableGitSubmodules": {
            value: data ? data.enableGitSubmodules : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Defines whether Git Fusion evaluates both the author's and pusher's Perforce write permissions during a push or evaluates only the pusher's permissions.

        */
        "ignoreAuthorPermissions": {
            value: data ? data.ignoreAuthorPermissions : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Enables you to trigger pre-flight commit scripts that enforce local policy for Git pushes. This can be especially useful if you have Perforce submit triggers that could reject a push and damage the repository.

        */
        "preflightCommit": {
            value: data ? data.preflightCommit : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Enables you to require that Git clone, pull, or fetch requests check the Perforce protections table for the puller's read permission on the files being pulled.

        */
        "readPermissionCheck": {
            value: data ? data.readPermissionCheck : undefined,
            enumerable: true,
            writable: true
        },
        /*
          If the Perforce service includes any changelists submitted by Git Fusion 13.2 or earlier, you can prevent unnecessary merge commits by setting this key to the number of the last changelist submitted before your site upgraded to a later version of Git Fusion.

        */
        "gitMergeAvoidanceAfterChangeNum": {
            value: data ? data.gitMergeAvoidanceAfterChangeNum : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Set the format for entering Perforce jobs in Git commit descriptions so that they are recognized by Git Fusion and appear in Perforce changelists as fixes.
          By default, job IDs whose string starts with "job" (as in job123456) are passed through to the changelist description and job field.
          Use this option if you want Git Fusion to recognize additional expressions, such as JIRA issue IDs.

        */
        "jobLookup": {
            value: data ? data.jobLookup : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Allow Git users to create new fully-populated depot branches within Perforce.

        */
        "depotBranchCreationEnable": {
            value: data ? data.depotBranchCreationEnable : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Restrict the authenticated Git pushers who are allowed to create new fully-populated depot branches, if depotBranchCreationEnable is enabled.

        */
        "depotBranchCreationP4group": {
            value: data ? data.depotBranchCreationP4group : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Tell Git Fusion where to create new fully-populated depot branches, if depotBranchCreationEnable is enabled.

          Default path is `//depot/[repo]/[git_branch_name]`.

        */
        "depotBranchCreationDepotPath": {
            value: data ? data.depotBranchCreationDepotPath : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Set how the depot path set in depotBranchCreationDepotPath should appear in Git.

          Enter a Perforce view specification that maps Perforce depot paths (left side) to Git work tree paths (right side).
          Perforce depot paths are relative to the root set in depotBranchCreationDepotPath.

          The default `... ...` maps every file under the depotBranchCreationDepotPath root to Git.
          Right side paths must match the right side for every other branch already defined within a repo.

        */
        "depotBranchCreationView": {
            value: data ? data.depotBranchCreationView : undefined,
            enumerable: true,
            writable: true
        },
        /*
          When Git reports a copy file action, store that action in Perforce as a p4 integ.
          Often set in tandem with enableGitFindRenames.

          No/Off/0%: Do not use Git's copy detection.
          Treat all possible file copy actions as p4 add actions.

          1%-100%: Use Git's copy detection. Value passed to git diff-tree --find-copies=n.

          Git Fusion also adds --find-copies-harder whenever adding --find-copies.

        */
        "enableGitFindCopies": {
            value: data ? data.enableGitFindCopies : undefined,
            enumerable: true,
            writable: true
        },
        /*
          When Git reports a rename (also called move) file action, store that in Perforce as a p4 move.
          Often set in tandem with enableGitFindCopies.

          `No`/`Off`/`0%`: Do not use Git's rename detection.
          Treat all possible file rename actions as independent p4 delete and p4 add actions.

          `1%`-`100%`: Use Git's rename detection.
          Value passed to git diff-tree --find-renames=n.

        */
        "enableGitFindRenames": {
            value: data ? data.enableGitFindRenames : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Enables you to convert Perforce stream import paths to Git submodules when you clone a Git Fusion repository.
          If set to Yes, you must also set either httpUrl or sshUrl.

        */
        "enableStreamImports": {
            value: data ? data.enableStreamImports : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The URL used by Git to clone a repository from Git Fusion over HTTP.
          This property is required if you want to use Perforce stream import paths as git submodules and you use HTTP(S).

        */
        "httpUrl": {
            value: data ? data.httpUrl : undefined,
            enumerable: true,
            writable: true
        },
        /*
          The "URL" used by Git to clone a repository from Git Fusion using SSH.
          This property is required if you want to use Perforce stream import paths as git submodules and you use SSH.

        */
        "sshUrl": {
            value: data ? data.sshUrl : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Defines whether Git Fusion pays attention to case when matching Git user email addresses to Perforce user account email addresses during the authorization check.

        */
        "emailCaseSensitivity": {
            value: data ? data.emailCaseSensitivity : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Defines the source that Git Fusion uses to identify the Perforce user associated with a Git push.

          Defaults to `git-email`.

          Use any one of the following values:

          - `git-email`: Use the email address of the Git author to look for a Perforce user account with the same email address. Git Fusion consults the p4gf_usermap file first, and if that fails to produce a match, it scans the Perforce user table.
          - `git-user`: Use the user.name field in the Git commit. This is the part of the author field before the email address.
          - `git-email-account`: Use the account portion of the Git author's email address. If the Git author's email value is `samwise@the_shire.com`, Git Fusion uses the Perforce account samwise.

          You can also tell Git Fusion to iterate through multiple source types until it finds a matching Perforce account. Specify the source types in order of precedence, separated by commas. For example: git-user, git-email-account, git-email.

        */
        "authorSource": {
            value: data ? data.authorSource : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Natural number representing the number of megabytes of disk space that can be consumed by any single repo.
          This value does not include the spaced consumed on the Perforce server.

        */
        "limitSpaceMb": {
            value: data ? data.limitSpaceMb : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Natural number representing the maximum number of commits allowed in a single push.

        */
        "limitCommitsReceived": {
            value: data ? data.limitCommitsReceived : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Natural number representing the maximum number of files allowed in a single push.

        */
        "limitFilesReceived": {
            value: data ? data.limitFilesReceived : undefined,
            enumerable: true,
            writable: true
        },
        /*
          Natural number representing the maximum number of megabytes allowed in a single push.

        */
        "limitMegabytesReceived": {
            value: data ? data.limitMegabytesReceived : undefined,
            enumerable: true,
            writable: true
        }
    });

}

module.exports = GitFusionRepoGlobalOverrides;
