var assert = require('assert');
var helix_web_services_client = require('../index.js');
var test_methods = require('./util/test_methods');
var string_methods = require('./util/string_methods');

var done = false;

test_methods.clientAsSuper(function (err, c) {
  if (err) {
    console.log("ERROR", err);
    assert(false, "login failed");
    return;
  }

  var api = c.createDefaultApi();

  var rand = string_methods.randomString();

  var depotName = "test-depot-" + rand;

  var depotCommand = new helix_web_services_client.models.DepotCommand({
    depot: depotName,
    type: "local",
    map: depotName + "/...",
    description: "test " + rand
  });

  api.serverDepotsPost("localhost", depotCommand, function(err, commandResponse) {
    if (err) {
      console.log("ERROR", err);
      assert(false);
    }

    api.serverDepotsGet("localhost", function(err, depots) {
      if (err) {
        console.log("ERROR", err);
        assert(false);
      }

      assert(depots.find(function(b) { return b.depot == depotName; }) != null,
        "did not locate new depot");

      api.serverDepotsDepotGet("localhost", depotName, function(err, saved) {
        if (err) {
          console.log("ERROR", err);
          assert(false);
        }

        assert(saved.description.trim() == depotCommand.description.trim());

        var toUpdate = new helix_web_services_client.models.DepotCommand({
          description: "update " + rand
        });

        api.serverDepotsDepotPatch("localhost", depotName, toUpdate, function (err) {
          if (err) {
            console.log("ERROR", err);
            assert(false);
          }

          api.serverDepotsDepotGet("localhost", depotName, function(err, updated) {
            if (err) {
              console.log("ERROR", err);
              assert(false);
            }

            assert(toUpdate.description.trim() == updated.description.trim());

            api.serverDepotsDepotDelete("localhost", depotName, function(err) {
              if (err) {
                console.log("ERROR", err);
                assert(false);
              }

              api.serverDepotsGet("localhost", function (err, depots2) {
                if (err) {
                  console.log("ERROR", err);
                  assert(false);
                }

                assert(depots2.find(function(b) { return b.depot == depotName; }) == null,
                  "depot was not deleted");

                done = true;
              });
            });
          });
        })
      });
    });
  });
});

function checkForDone() {
  if (!done) {
    setTimeout(checkForDone, 500);
  }
}

setTimeout(checkForDone, 0);
