require 'date'

module HelixWebServices
  # A changelist specification.\n\nFor more information see the [command reference](https://www.perforce.com/perforce/doc.current/manuals/cmdref/p4_change.html).
  class ChangeCommand
    # Contains the changelist number if editing an existing changelist, or `new` if creating a new changelist.
    attr_accessor :change

    # Name of current client workspace
    attr_accessor :client

    # Date the changelist was last modified.
    attr_accessor :date

    # Name of the change owner.\n\nThe owner of an empty pending changelist (that is, a pending changelist without any files in it) can transfer ownership of the changelist to another existing user either by editing this field, or by using the -U user option.\n\nThe specified owner does not have to be a Perforce user.\nYou might want to use an arbitrary name if the user does not yet exist, or if you have deleted the user and need a placeholder until you can assign the spec to a new user.
    attr_accessor :user

    # `pending`, `shelved`, `submitted`, or `new`.\nNot editable by the user.\n\nThe status is `new` when the changelist is created, `pending` when it has been created but has not yet been submitted to the depot, `shelved` when its contents are shelved, and `submitted` when its contents have been stored in the depot.
    attr_accessor :status

    # Textual description of changelist.\n\nIf you do not have access to a restricted changelist, the description is replaced with a \"no permission\" message.
    attr_accessor :description

    # A list of jobs that are fixed by this changelist.
    attr_accessor :jobs

    # Type of change: `restricted` or `public`.\n\nThe Type: field can be used to hide the change or its description from users.\nA shelved or committed change (as denoted in the Status: field) that is restricted is accessible only to users who own the change or have list permission to at least one file in the change.\n\nPublic changes are displayed without restrictions.\n\nBy default, changelists are public.\nA Perforce superuser can set the default changelist type (for changelists created after the configurable is set) by setting the defaultChangeType configurable.
    attr_accessor :type

    # The list of files submitted in this changelist.
    attr_accessor :files

    # Displays the name of the user who ran the p4 fetch, p4 push, or p4 unzip command that imported this change into the server.\n\nThis field is primarily useful for distributed versioning (DVCS) scenarios, in which changelists are copied from one server to another, and help you correlate the changelist’s basic identity as it is copied.\n\nIn such configurations, Perforce recommends using the submit.identity configurable to enable automatic generation of changelist identities by the p4 submit.
    attr_accessor :imported_by

    # Contains a label which uniquely identifies this changelist across all servers where it has been fetched, pushed, or unzipped.\n\nThis field is primarily useful for distributed versioning (DVCS) scenarios, in which changelists are copied from one server to another, and help you correlate the changelist’s basic identity as it is copied.\n\nIn such configurations, Perforce recommends using the submit.identity configurable to enable automatic generation of changelist identities by the p4 submit.
    attr_accessor :identify

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        
        :'change' => :'change',
        
        :'client' => :'client',
        
        :'date' => :'date',
        
        :'user' => :'user',
        
        :'status' => :'status',
        
        :'description' => :'description',
        
        :'jobs' => :'jobs',
        
        :'type' => :'type',
        
        :'files' => :'files',
        
        :'imported_by' => :'importedBy',
        
        :'identify' => :'identify'
        
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'change' => :'String',
        :'client' => :'String',
        :'date' => :'DateTime',
        :'user' => :'String',
        :'status' => :'String',
        :'description' => :'String',
        :'jobs' => :'Array<String>',
        :'type' => :'String',
        :'files' => :'Array<String>',
        :'imported_by' => :'String',
        :'identify' => :'String'
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes[:'change']
        self.change = attributes[:'change']
      end
      
      if attributes[:'client']
        self.client = attributes[:'client']
      end
      
      if attributes[:'date']
        self.date = attributes[:'date']
      end
      
      if attributes[:'user']
        self.user = attributes[:'user']
      end
      
      if attributes[:'status']
        self.status = attributes[:'status']
      end
      
      if attributes[:'description']
        self.description = attributes[:'description']
      end
      
      if attributes[:'jobs']
        if (value = attributes[:'jobs']).is_a?(Array)
          self.jobs = value
        end
      end
      
      if attributes[:'type']
        self.type = attributes[:'type']
      end
      
      if attributes[:'files']
        if (value = attributes[:'files']).is_a?(Array)
          self.files = value
        end
      end
      
      if attributes[:'importedBy']
        self.imported_by = attributes[:'importedBy']
      end
      
      if attributes[:'identify']
        self.identify = attributes[:'identify']
      end
      
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          change == o.change &&
          client == o.client &&
          date == o.date &&
          user == o.user &&
          status == o.status &&
          description == o.description &&
          jobs == o.jobs &&
          type == o.type &&
          files == o.files &&
          imported_by == o.imported_by &&
          identify == o.identify
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [change, client, date, user, status, description, jobs, type, files, imported_by, identify].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      when :Object
        value
      else # model
        _model = HelixWebServices.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
