require 'date'

module HelixWebServices
  class ChangesCommand
    # The changelist ID
    attr_accessor :change

    # Last modification time of the changelist
    attr_accessor :date

    # The owner of the changelist
    attr_accessor :user

    # Name of current client workspace.
    attr_accessor :client

    # `pending`, `shelved`, `submitted`, or `new`.\nNot editable by the user.\n\nThe status is `new` when the changelist is created, `pending` when it has been created but has not yet been submitted to the depot, `shelved` when its contents are shelved, and `submitted` when its contents have been stored in the depot.
    attr_accessor :status

    # Type of change: `restricted` or `public`.\n\nThe Type: field can be used to hide the change or its description from users.\nA shelved or committed change (as denoted in the Status: field) that is restricted is accessible only to users who own the change or have list permission to at least one file in the change.\n\nPublic changes are displayed without restrictions.\n\nBy default, changelists are public.\nA Perforce superuser can set the default changelist type (for changelists created after the configurable is set) by setting the defaultChangeType configurable.
    attr_accessor :type

    # Depot paths affected by this changelist
    attr_accessor :_path

    # Textual description of changelist.\n\nIf you do not have access to a restricted changelist, the description is replaced with a \"no permission\" message.
    attr_accessor :description

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        
        :'change' => :'change',
        
        :'date' => :'date',
        
        :'user' => :'user',
        
        :'client' => :'client',
        
        :'status' => :'status',
        
        :'type' => :'type',
        
        :'_path' => :'path',
        
        :'description' => :'description'
        
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'change' => :'String',
        :'date' => :'DateTime',
        :'user' => :'String',
        :'client' => :'String',
        :'status' => :'String',
        :'type' => :'String',
        :'_path' => :'String',
        :'description' => :'String'
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes[:'change']
        self.change = attributes[:'change']
      end
      
      if attributes[:'date']
        self.date = attributes[:'date']
      end
      
      if attributes[:'user']
        self.user = attributes[:'user']
      end
      
      if attributes[:'client']
        self.client = attributes[:'client']
      end
      
      if attributes[:'status']
        self.status = attributes[:'status']
      end
      
      if attributes[:'type']
        self.type = attributes[:'type']
      end
      
      if attributes[:'path']
        self._path = attributes[:'path']
      end
      
      if attributes[:'description']
        self.description = attributes[:'description']
      end
      
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          change == o.change &&
          date == o.date &&
          user == o.user &&
          client == o.client &&
          status == o.status &&
          type == o.type &&
          _path == o._path &&
          description == o.description
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [change, date, user, client, status, type, _path, description].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      when :Object
        value
      else # model
        _model = HelixWebServices.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
